from .find import find
from .pick import pick
from .phot import phot
from .create_psf import create_psf
from .sub_fits import sub_fits
from .allstar import allstar
from .daomatch import daomatch
from .create_master import create_master


# Strings descriptivos
opt_daophot_labels = {
    're': 'Readout Noise',
    'ga': 'Gain',
    'lo': 'Low Good',
    'hi': 'High Good',
    'fw': 'FWHM of Star',
    'th': 'Threshold',
    'ls': 'Low Sharpness',
    'lr': 'Low Roundness',
    'hs': 'High Sharpness',
    'hr': 'High Roundness',
    'wa': 'Watch Progress',
    'fi': 'Fitting Radius',
    'ps': 'PSF Radius',
    'va': 'Variable PSF',
    'an': 'Analytic PSF',
    'ex': 'PSF Cleaning',
    'us': 'us (DEPRECATED)',
    'pr': 'Profile Error',
    'pe': 'Percent Error',
}

opt_photo_labels = {
    "A1": "Apert Radii 1",
    "A2": "Apert Radii 2",
    "A3": "Apert Radii 3",
    "A4": "Apert Radii 4",
    "A5": "Apert Radii 5",
    "A6": "Apert Radii 6",
    "A7": "Apert Radii 7",
    "A8": "Apert Radii 8",
    "A9": "Apert Radii 9",
    "AA": "Apert Radii 10",
    "AB": "Apert Radii 11",
    "AC": "Apert Radii 12",
    "IS": "InnerSky Radii",
    "OS": "OuterSky Radii"
}

opt_allstar_labels = {
    "fi": "Fitting Radius",
    "re": "Recenter Stars",
    "wa": "Watch Progress",
    "pe": "Percent Error",
    "ce": "Clip Exponent",
    "cr": "Clip Range",
    "ma": "Max Group Size",
    "pr": "Profile Error",
    "is": "InnerSky Radii",
    "os": "OuterSky Radii"
}

info_docstrings = {
"INFO_re": """
READ NOISE: 
El ruido de lectura, en números de datos (DN), de una sola exposición 
realizada con tu detector. Más tarde, el software permitirá especificar 
si un marco de datos es la suma o el promedio de múltiples exposiciones. 
Si tienes un subdirectorio separado para datos de una noche o corrida 
de observación, el ruido de lectura debe especificarse solo una vez 
(en el archivo DAOPHOT.OPT).
""",

"INFO_ga": """
GAIN: 
El factor de ganancia de tu detector, en fotones o electrones por número 
de datos (DN). Al igual que con el ruido de lectura, debes especificar 
la ganancia correspondiente a una sola exposición. Se puede hacer un 
ajuste más tarde para los marcos que son la media o la suma de varias 
exposiciones. Los valores por defecto son deliberadamente inválidos. 
Debes ingresar los valores correctos en un archivo DAOPHOT.OPT o el 
programa te pedirá que lo hagas repetidamente.
""",

"INFO_lo": """
LOW GOOD DATUM: 
El nivel, en desviaciones estándar por debajo del valor medio del cielo 
en el marco, por debajo del cual el programa considera un píxel como 
defectuoso. Si el fondo es plano en todo el marco, puedes establecer 
un límite ajustado, como 5σ. Si hay un gradiente de fondo fuerte, 
necesitarás un límite más generoso, tal vez 10σ o más, para evitar 
rechazar píxeles de cielo legítimos en las partes del marco donde el 
fondo es débil. El uso inteligente del comando DUMP y/o una visualización 
de imagen te ayudará a decidir.
""",

"INFO_hi": """
HIGH GOOD DATUM: 
El nivel, en números de datos (DN), por encima del cual un valor de 
píxel se considera defectuoso. Esto difiere del "LOW GOOD DATUM". 
El "LOW GOOD DATUM" se define como un cierto número de desviaciones 
estándar por debajo del valor medio del cielo del marco, por lo que 
necesita especificarse solo una vez si todos tus marcos tienen fondos 
comparables. El "HIGH GOOD DATUM" se especifica como un único número 
fijo que representa el nivel absoluto en números de datos donde el 
detector se vuelve no lineal o se satura. Como tus datos han sido 
sustrayendo el nivel de sesgo y normalizados, este número no será 32767, 
sino algo más bajo.
""",

"INFO_fw": """
FWHM: 
El FWHM aproximado, en píxeles, de los objetos para los cuales el 
algoritmo FIND está optimizado. Este parámetro determina el ancho de 
la función gaussiana y el tamaño del arreglo con el cual tu imagen es 
numéricamente convolucionada por FIND. Si las condiciones durante tu 
corrida fueron razonablemente constantes, un solo valor debería ser 
suficiente para todos tus marcos.
""",

"INFO_th": """
THRESHOLD: 
El nivel de significancia, en desviaciones estándar, que deseas que 
el programa utilice para decidir si un aumento positivo en el brillo 
es real. Normalmente, un valor alrededor de 4σ es bueno, pero puede 
que quieras establecerlo un poco más alto para la primera pasada.
""",

"INFO_ls": """
LOW SHARPNESS CUTOFF: 
El valor mínimo para la nitidez de un aumento de brillo que FIND 
considera una estrella real. Esto está destinado a eliminar píxeles 
defectuosos y también puede ayudar a eliminar galaxias de baja 
luminosidad. En la mayoría de los casos, los valores por defecto 
proporcionados en el programa son adecuados, pero si deseas afinar 
los valores, aquí están.
""",

"INFO_lr": """
HIGH SHARPNESS CUTOFF: 
El valor máximo para la nitidez de un aumento de brillo que FIND 
considera una estrella real. Esto está destinado a eliminar píxeles 
defectuosos y puede ayudar a eliminar algunas galaxias de baja 
luminosidad. Nuevamente, los valores predeterminados en el programa 
suelen ser adecuados para la mayoría de los casos, pero puedes 
ajustarlos según sea necesario.
""",

"INFO_hs": """
LOW ROUNDNESS CUTOFF: 
El valor mínimo para la redondez de un aumento de brillo que FIND 
considera una estrella real. Esto está destinado a eliminar filas y 
columnas defectuosas, y también puede rechazar algunas galaxias de 
borde. En general, los valores predeterminados en el programa son 
adecuados, pero pueden ser ajustados para casos especiales.
""",

"INFO_hr": """
HIGH ROUNDNESS CUTOFF: 
El valor máximo para la redondez de un aumento de brillo que FIND 
considera una estrella real. Esto está destinado a eliminar filas y 
columnas defectuosas, y puede rechazar algunas galaxias de borde. 
Los valores predeterminados en el programa suelen ser adecuados para 
la mayoría de los casos, pero puedes ajustarlos si es necesario.
""",

"INFO_wa": """
WATCH PROGRESS: 
Indica si se deben mostrar los resultados en el terminal de la 
computadora en tiempo real mientras se calculan. Mostrar los 
resultados puede mantenerte entretenido mientras se realizan las 
reducciones, pero puede ralentizar el tiempo de ejecución y, en modo 
por lotes, llenar excesivamente tu archivo de registro.
""",

"INFO_fi": """
FITTING RADIUS: 
Este parámetro define el área circular dentro de la cual se usarán los 
píxeles para realizar los ajustes de perfil en PEAK y NSTAR. La 
función de dispersión puntual se ajusta para determinar la posición 
y el brillo de cada estrella en tu programa. Solo los píxeles dentro 
de un radio de ajuste del centroide se usarán en el ajuste. También se 
usa para ajustar la primera aproximación analítica a la PSF para las 
estrellas PSF y contribuye en menor medida a la determinación de cuando 
las estrellas se superponen "significativamente". Bajo circunstancias 
normales, este radio debe ser del orden del FWHM de una imagen 
estelar. Cuando la aglomeración es extremadamente 
severa, puede ser ventajoso usar un valor algo menor que esto. Por otro 
lado, si se sabe que la PSF varía a través del marco, aumentar el radio 
de ajuste más allá del FWHM puede mejorar la precisión fotométrica, siempre 
que el campo no esté horrible.

Consejos:
    - Debería ser igual o un poco mayor que el FWHM para una fotometría 
      óptima.
""",

"INFO_ps": """
PSF RADIUS: 
El radio, en píxeles, del círculo dentro del cual se definirá la función 
de dispersión puntual (PSF). Debe ser algo mayor que el radio de la 
estrella más brillante de interés, como se mediría en tu visualización 
de imagen. Si al final de tus reducciones (ver §B arriba), notas que las 
imágenes substraídas de tus estrellas brillantes están rodeadas de halos 
luminosos con bordes internos agudos, entonces tu radio PSF es demasiado 
pequeño. Por otro lado, el tiempo de CPU requerido para las reducciones 
de ajuste de perfil es una función fuerte del radio PSF, por lo que es 
contraproducente hacer que este parámetro sea demasiado grande.

Consejos:
    - El valor de PSF debería ser 3 o 4 veces mayor que FWHM para 
      garantizar que las alas del PSF estén suficientemente alejadas.
""",

"INFO_va": """
VARIABLE PSF: 
El grado de complejidad con el que se modelará la PSF. En sus inicios, 
DAOPHOT Classic permitía solo una forma para el modelo PSF: una 
aproximación analítica gaussiana, más una tabla de corrección empírica 
del modelo analítico a la PSF "real". Esto corresponde a VARIABLE PSF = 0. 
Más tarde, se añadió la posibilidad de una PSF que varía linealmente con 
la posición en el marco; esto es VARIABLE PSF = 1. DAOPHOT II ahora permite 
dos posibilidades adicionales: una PSF que varía cuadráticamente con la 
posición en el marco (VARIABLE PSF = 2), y un modelo PSF puramente analítico, 
sin tabla de corrección empírica, como en ROMAFOT (VARIABLE PSF = -1). Es 
probable que sea mejor dejarlo en 0.0 (= "Constante") hasta que estés 
seguro de lo que estás haciendo.

Consejos:
    - Comienza siempre con VAR=0 (PSF constante) y revisa los resultados. 
      Modelos de PSF más complejos requieren más estrellas definidas.
""",

"INFO_an": """
ANALYTIC MODEL PSF: 
DAOPHOT Classic siempre usó una función gaussiana como una aproximación 
analítica a la PSF. DAOPHOT II permite una serie de alternativas que se 
discutirán a continuación bajo el comando PSF.

Consejos:
    - Para imágenes CCD normales, AN=1 (Gaussiana) suele ser adecuado.
    - Para imágenes HST/WFPC2, AN=4 (Lorentziana) es más adecuado.
    - Consulta el manual para más detalles sobre otras funciones.
""",

"INFO_ex": """
EXTRA PSF CLEANING PASSES: 
DAOPHOT II ahora puede reconocer y reducir el peso de píxeles claramente 
discordantes mientras genera una tabla de corrección PSF. Al igual que con 
FIND, puedes reducir el peso de píxeles discordantes mediante el comando 
PSF o durante el proceso de ajuste de perfil. El número de veces que el 
programa pasará por los datos es especificado por EXTRA PSF CLEANING PASSES. 
El valor típico es de 5 pasadas, aunque en la mayoría de los casos, 
establecer esto en 0 es suficiente.
""",

"INFO_us": """
FRACTIONAL PIXEL EXPANSION: 
Este parámetro no está implementado en DAOPHOT II. Déjalo tal como está. 
Es un remanente de DAOPHOT Classic.
""",

"INFO_pr": """
PROFILE ERROR: 
El error asociado a la PSF debido a la falta de precisión infinita en la 
función. La incertidumbre de los parámetros de perfil se ajusta al error 
específico. El error aumenta linealmente con la intensidad de la estrella 
y disminuye con la cuarta potencia del FWHM. Los valores por defecto 
proporcionados en el programa deberían ser adecuados, aunque en algunos 
casos puede ser necesario ajustar este parámetro para garantizar una 
precisión aceptable.
""",

"INFO_pe": """
PERCENT ERROR: 
Este parámetro especifica el valor de incertidumbre en la estructura fina 
del campo plano. Aumenta linealmente con la intensidad de la estrella más 
el fondo. La precisión del ajuste fotométrico y del modelo PSF está 
asociada a este parámetro. Los valores por defecto en el programa son 
adecuados en la mayoría de los casos, pero puede ser necesario ajustarlos 
para lograr una precisión más fina en algunos casos.
""",

"INFO_A1": """
RADIO DE APERTURA A1:
El radio de apertura en píxeles para la primera configuración específica. 
Este parámetro define el tamaño del área circular alrededor de cada estrella 
donde se mide el flujo de luz. La elección de un radio adecuado es crucial 
para la precisión de la fotometría. Un radio más grande captura más luz, 
pero también puede incluir más ruido, mientras que un radio más pequeño 
puede perder parte de la luz de la estrella. Ajusta el radio en función del 
tamaño de las estrellas y las condiciones de imagen para obtener medidas 
precisas y fiables.
""",

"INFO_A2": """
RADIO DE APERTURA A2:
El radio de apertura en píxeles para la segunda configuración específica. 
Este radio permite definir otro tamaño de apertura circular para medir el 
flujo de luz de las estrellas. Puedes utilizar radios consecutivos para 
comparar cómo cambia la medición del flujo con diferentes tamaños de apertura.
""",

"INFO_A3": """
RADIO DE APERTURA A3:
El radio de apertura en píxeles para la tercera configuración. Este radio 
define un área circular adicional para medir el flujo de luz de las estrellas. 
Consulta RADIO DE APERTURA A1 para más detalles sobre cómo elegir el radio 
adecuado.
""",

"INFO_A4": """
RADIO DE APERTURA A4:
El radio de apertura en píxeles para la cuarta configuración específica. 
Utiliza este radio para ajustar la medición del flujo estelar. Referencia 
RADIO DE APERTURA A1 para más información sobre el ajuste de los radios de apertura.
""",

"INFO_A5": """
RADIO DE APERTURA A5:
El radio de apertura en píxeles para la quinta configuración. Define otro 
tamaño de apertura circular para las mediciones estelares. Consulta RADIO 
DE APERTURA A1 para detalles sobre cómo estos radios afectan la precisión 
de las mediciones.
""",

"INFO_A6": """
RADIO DE APERTURA A6:
El radio de apertura en píxeles para la sexta configuración. Este radio 
especifica un área adicional para medir la luz de las estrellas. Refiérete 
a RADIO DE APERTURA A1 para ajustar los radios de apertura de acuerdo con 
tus necesidades.
""",

"INFO_A7": """
RADIO DE APERTURA A7:
El radio de apertura en píxeles para la séptima configuración. Utiliza 
este radio para definir un área circular para las mediciones estelares. 
Consulta RADIO DE APERTURA A1 para más detalles sobre la elección del radio 
adecuado.
""",

"INFO_A8": """
RADIO DE APERTURA A8:
El radio de apertura en píxeles para la octava configuración específica. 
Este radio define un área circular adicional para medir el flujo estelar. 
Refiérete a RADIO DE APERTURA A1 para información sobre cómo ajustar el radio 
para obtener medidas precisas.
""",

"INFO_A9": """
RADIO DE APERTURA A9:
El radio de apertura en píxeles para la novena configuración. Define un 
tamaño de apertura circular para medir la luz de las estrellas. Consulta 
RADIO DE APERTURA A1 para ajustar los radios de apertura y mejorar la 
precisión de las mediciones.
""",

"INFO_AA": """
RADIO DE APERTURA AA:
El radio de apertura en píxeles para la décima configuración específica. 
Utiliza este radio para definir un área circular adicional para las 
mediciones estelares. Referencia RADIO DE APERTURA A1 para detalles sobre 
cómo estos radios afectan tus mediciones.
""",

"INFO_AB": """
RADIO DE APERTURA AB:
El radio de apertura en píxeles para la undécima configuración. Define el 
tamaño de la apertura para medir el flujo de luz de las estrellas. Refiérete 
a RADIO DE APERTURA A1 para información adicional sobre la elección del radio.
""",

"INFO_AC": """
RADIO DE APERTURA AC:
El radio de apertura en píxeles para la duodécima configuración. Este radio 
especifica un área circular para la medición de la luz estelar. Consulta 
RADIO DE APERTURA A1 para ajustar el radio según las características de tus 
imágenes y estrellas.
""",

"INFO_a_is": """
INNER SKY RADIUS:
El RADIO INTERIOR DEL CIELO define el tamaño del área circular interna 
utilizada para calcular el valor del fondo del cielo alrededor de las 
estrellas. Este parámetro debe ser menor que el RADIO EXTERIOR DEL CIELO. 
Un valor pequeño (como 0.00) puede ser útil para minimizar la influencia del 
ruido cerca del centro de la estrella. Sin embargo, un valor muy pequeño puede 
llevar a una estimación menos precisa del fondo. Un valor típico podría ser 
igual al RADIO DE APERTURA o ligeramente mayor.

Consejos:
- Establece IS cerca del RADIO DE APERTURA o un poco mayor para una estimación 
    precisa del fondo del cielo.
- Un valor pequeño minimiza la influencia del ruido, pero asegúrate de que 
    sea suficientemente grande para captar una muestra representativa del fondo 
    del cielo.
""",

"INFO_a_os": """
OUTER SKY RADIUS:
El RADIO EXTERIOR DEL CIELO define el tamaño del área circular externa 
utilizada para calcular el valor del fondo del cielo. Este parámetro debe 
ser mayor que el RADIO INTERIOR DEL CIELO. Debe ser grande en comparación 
con el RADIO DE APERTURA para incluir suficientes datos del fondo del cielo y 
evitar influencias de variaciones locales. Un valor típico es comparable al 
RADIO DE APERTURA o un poco más grande.

Consejos:
- Establece OS significativamente mayor que IS para asegurar una buena 
    estimación del fondo del cielo.
- Un valor mayor que el RADIO DE APERTURA y comparable al RADIO DE APERTURA 
    suele ser adecuado para evitar influencias locales en el cálculo del fondo 
    del cielo.
""",

"INFO_sumaver": """
Number of frames averaged, summed:
Este parámetro indica el número de exposiciones independientes que se han 
promediado o sumado para crear el marco que estás a punto de reducir. La 
información proporcionada se utiliza para ajustar adecuadamente el ruido de 
lectura y la ganancia en el procesamiento de la imagen. El valor se introduce 
en un formato específico que indica cuántos marcos se han promediado y cuántos 
se han sumado. Por ejemplo:
- "5,1" significa que se promediaron cinco exposiciones y se sumaron.
- "1,5" indica que se sumaron cinco exposiciones y no se promediaron.
- "3,2" significa que se promediaron tres sumas de dos exposiciones cada una.
Si el marco representa la mediana de varias exposiciones independientes, 
se debe ingresar como si fuera el promedio de dos tercios del número de marcos. 
Por ejemplo, la mediana de tres imágenes se ingresaría como "2,1".

Consejos:
- Introduce el número de marcos promediados y sumados de acuerdo con el proceso 
    de reducción específico que seguiste.
- Usa el formato apropiado para reflejar con precisión el tratamiento de las 
    exposiciones en tus datos.
""",

"INFO_minmag": """
Minimum Magnitude:
Este parámetro define el límite inferior para la magnitud de las estrellas 
que serán incluidas en el análisis. Las estrellas con magnitudes inferiores 
a este valor serán consideradas para el ajuste y el cálculo. Establecer un 
valor adecuado para la magnitud mínima es crucial para asegurarse de que 
todas las estrellas relevantes se incluyan en el análisis sin introducir 
ruido innecesario.

Consejos:
- Ajusta el valor de la magnitud mínima para incluir todas las estrellas que 
    esperas analizar, excluyendo aquellas que podrían ser demasiado tenues o 
    no relevantes.
- Establece este parámetro con base en la sensibilidad de tu imagen y el rango 
    de magnitudes de las estrellas presentes en tu marco.
""",

"INFO_a_fi": """
FITTING RADIUS:
El RADIO DE AJUSTE determina el tamaño del área circular alrededor de 
cada estrella que se utiliza para ajustar la función de dispersión de 
puntos (PSF). Este parámetro es crucial para la precisión del ajuste, ya 
que define el rango dentro del cual se considera la información de la 
estrella para el ajuste. Un RADIO DE AJUSTE mayor incluye más datos en el 
ajuste, lo que puede mejorar la precisión pero también puede hacer que el 
ajuste sea menos sensible a variaciones locales en el fondo. Un RADIO DE 
AJUSTE menor puede ser más sensible a las variaciones locales y puede dar 
lugar a un ajuste más fino. En general, un valor típico para el RADIO DE 
AJUSTE es de 2.50 píxeles, pero puede necesitar ajustes dependiendo de la 
densidad de las estrellas y la calidad de la imagen.

Consejos:
- Para imágenes con estrellas no muy densas y en buena calidad, un valor 
    de 2.50 píxeles es un buen punto de partida.
- Ajusta el radio según la densidad estelar y la calidad de la imagen. Un 
    RADIO DE AJUSTE mayor puede ser necesario si las estrellas están muy 
    separadas, mientras que un valor menor puede ser más adecuado para 
    estrellas muy juntas.
""",

"INFO_a_re": """
REDETERMINE CENTROIDS:
Este parámetro indica si se deben recalcular las posiciones de los 
centroides de las estrellas durante el ajuste. Un valor de 1.00 significa 
que las posiciones de las estrellas se recalcularán para mejorar la 
precisión, mientras que un valor de 0.00 asume que las posiciones de las 
estrellas son conocidas con precisión y solo se ajustarán las magnitudes. 
Si se tiene confianza en las posiciones de las estrellas, se puede fijar 
a 0.00 para ahorrar tiempo de cálculo, pero si se necesita la máxima 
precisión, se debe usar 1.00.

Consejos:
- Usa 1.00 si necesitas ajustar tanto las posiciones como las magnitudes 
    para obtener la máxima precisión.
- Configura en 0.00 si las posiciones de las estrellas ya son precisas y 
    solo necesitas ajustar las magnitudes. Esto puede ahorrar tiempo de 
    procesamiento si se tiene una lista de estrellas precisa.
""",

"INFO_a_wa": """
WATCH PROGRESS:
El parámetro OBSERVAR PROGRESO controla la cantidad de información de 
progreso que se muestra durante la ejecución del programa en la terminal. 
Un valor de 1.00 activa la visualización de mensajes detallados sobre el 
progreso del procesamiento, lo que puede ser útil para monitorear la 
ejecución en tiempo real y detectar problemas. Un valor de 0.00 desactiva 
estos mensajes, lo que puede acelerar la ejecución si se está ejecutando 
en un entorno de producción o si no es necesario monitorear el progreso.

Consejos:
- Esta función no es muy útil en la interfaz, pero es bueno no olvidarla 
    si usas DAOPHOT.
- Si necesitas monitorear el progreso del procesamiento, usa 1.00.
- Para ejecuciones en segundo plano o en producción, especialmente si no 
    es necesario ver detalles del progreso, configura este valor en 0.00 
    para mejorar la eficiencia.
""",

"INFO_a_pe": """
PERCENT ERROR:
En el cálculo del error estándar esperado para el valor de brillo en un 
píxel, el programa utiliza el ruido de lectura y las estadísticas de 
Poisson del número esperado de fotones. Este parámetro permite especificar 
un valor particular para la incertidumbre de la estructura fina del campo 
plano. El error de Poisson aumenta con la raíz cuadrada de la intensidad, 
mientras que el ERROR PORCENTAJE aumenta linealmente con la intensidad del 
(estrella + cielo). Se refiere a la granularidad del desajuste del campo 
plano utilizado para calibrar las imágenes del programa.

Consejos:
- Deja este parámetro por defecto hasta que tengas una comprensión 
    avanzada del ajuste.
- Ajustar este parámetro puede afectar la precisión del ajuste en función 
    de la granularidad del campo plano.
""",

"INFO_a_ce": """
CLIPPING EXPONENT:
El EXPONENTE DE RECORTE es utilizado en la fórmula de recorte de datos para 
manejar datos atípicos. Este parámetro define el exponente en la fórmula 
de recorte y tiene un impacto significativo en la forma en que se manejan 
los datos atípicos. Un valor de 6.00 es el valor predeterminado, pero este 
parámetro puede ajustarse según la calidad y la distribución de los datos. 
Experimentar con diferentes valores puede ayudar a optimizar los resultados, 
aunque puede requerir mucho tiempo de CPU.

Consejos:
- Los valores predeterminados (6.00) suelen funcionar bien, pero puedes 
    ajustar el EXPONENTE DE RECORTE si encuentras que los datos atípicos 
    están afectando los resultados.
- Experimenta con diferentes valores si estás dispuesto a invertir tiempo 
    en la optimización, pero ten en cuenta que esto puede aumentar el tiempo 
    de procesamiento.
- Para una configuración más conservadora, establece el valor en 0.0 para 
    desactivar el recorte de datos atípicos.
""",

"INFO_a_cr": """
CLIPPING RANGE:
El RANGO DE RECORTE define el umbral en la fórmula de recorte que determina 
cuánto peso se le da a los datos atípicos. Un valor de 2.50 se usa por 
defecto, lo que significa que los residuos mayores a 2.5 veces la desviación 
estándar reciben la mitad del peso. Ajustar este parámetro puede ayudar a 
mejorar la robustez del ajuste frente a datos atípicos, pero también puede 
aumentar el tiempo de procesamiento. Valores de rango más altos pueden 
permitir una mayor influencia de los datos atípicos, mientras que valores 
más bajos pueden descartar más datos.

Consejos:
- Los valores predeterminados (2.50) suelen ser adecuados, pero puedes 
    ajustar el RANGO DE RECORTE si los datos atípicos están afectando la 
    precisión del ajuste.
- Ajusta este parámetro con cuidado, ya que valores más altos pueden 
    incluir más datos atípicos y valores más bajos pueden eliminar datos útiles.
""",

"INFO_a_ma": """
MAXIMUM GROUP SIZE:
El parámetro TAMAÑO MÁXIMO DEL GRUPO define el número máximo de estrellas 
permitidas en un solo grupo durante el procesamiento. Cuando el número de 
estrellas en un grupo excede este límite, ALLSTAR intentará dividir el grupo 
en subgrupos más pequeños. Aumentar el valor de TAMAÑO MÁXIMO DEL GRUPO puede 
reducir el número de estrellas descartadas debido a agrupaciones demasiado 
grandes, pero también puede aumentar el tiempo de procesamiento. Un valor 
típico podría ser 50, pero es importante ajustar este parámetro según la 
densidad de estrellas en las imágenes y la capacidad de procesamiento 
disponible.

Consejos:
- Para imágenes con alta densidad estelar, comienza con 50 y ajusta según 
    sea necesario.
- Valores más altos permiten retener más estrellas en el análisis, pero 
    también aumentan el tiempo de procesamiento. Si encuentras que muchas 
    estrellas están siendo descartadas, considera aumentar este valor.
""",

"INFO_a_pr": """
PROFILE ERROR:
Al ajustar la función de dispersión de puntos a las imágenes estelares 
reales, hay errores debido a que la función de dispersión no es conocida 
con precisión infinita. Este parámetro define la amplitud de esta 
contribución al modelo de ruido; el ERROR DE PERFIL aumenta linealmente 
con la intensidad de la estrella sola (sin cielo) e inversamente a la 
cuarta potencia del ancho completo a la mitad del máximo. Por lo tanto, 
este error crece en importancia relativa al ERROR PORCENTAJE a medida que 
mejora la visión.

Consejos:
- Deja este parámetro por defecto hasta que tengas una comprensión 
    avanzada del ajuste.
- Ajustar el ERROR DE PERFIL puede ser necesario si la calidad de la imagen 
    mejora significativamente.
""",

"INFO_a_is": """
INNER SKY RADIUS:
El RADIO INTERIOR DEL CIELO define el tamaño del área circular interna 
utilizada para calcular el valor del fondo del cielo alrededor de las 
estrellas. Este parámetro debe ser menor que el RADIO EXTERIOR DEL CIELO. 
Un valor pequeño (como 0.00) puede ser útil para minimizar la influencia del 
ruido cerca del centro de la estrella. Sin embargo, un valor muy pequeño puede 
llevar a una estimación menos precisa del fondo. Un valor típico podría ser 
igual al RADIO DE AJUSTE o ligeramente mayor.

Consejos:
- Establece IS cerca del RADIO DE AJUSTE o un poco mayor para una estimación 
    precisa del fondo del cielo.
- Un valor pequeño minimiza la influencia del ruido, pero asegúrate de que 
    sea suficientemente grande para captar una muestra representativa del fondo 
    del cielo.
""",

"INFO_a_os": """
OUTER SKY RADIUS:
El RADIO EXTERIOR DEL CIELO define el tamaño del área circular externa 
utilizada para calcular el valor del fondo del cielo. Este parámetro debe 
ser mayor que el RADIO INTERIOR DEL CIELO. Debe ser grande en comparación 
con el RADIO DE AJUSTE para incluir suficientes datos del fondo del cielo y 
evitar influencias de variaciones locales. Un valor típico es comparable al 
RADIO DE DISPERSIÓN DEL PSF o un poco más grande.

Consejos:
- Establece OS significativamente mayor que IS para asegurar una buena 
    estimación del fondo del cielo.
- Un valor mayor que el RADIO DE AJUSTE y comparable al RADIO DE DISPERSIÓN 
    DEL PSF suele ser adecuado para evitar influencias locales en el cálculo 
    del fondo del cielo.
"""

}



from ..misc_tools import check_file, temp_mkdir, move_file_noreplace
import os
import tempfile
import shutil
from .docker_handler import run_proc, docker_run


def allstar(in_fits, in_psf, in_ap, in_daophot, in_allstar,
                out_als, out_fits, RE=True, verbose=True, 
                	use_docker=None, working_dir=".") -> [".als", ".fits"]:
    try:
        # Copiar archivos necesarios a la carpeta temporal
        filename = os.path.splitext(os.path.basename(in_fits))[0]
        out_fits_filename = "allstar_substracted"
        out_als_filename = "allstar_out_file"
        # Crear carpeta temporal
        temp_dir      = os.path.abspath(temp_mkdir(os.path.join(working_dir, f"{filename}_ALLSTAR_0")))
        temp_fits     = os.path.join(temp_dir, os.path.basename(in_fits))
        temp_psf      = os.path.join(temp_dir, os.path.basename(in_psf))
        temp_ap       = os.path.join(temp_dir, os.path.basename(in_ap))
        temp_daophot  = os.path.join(temp_dir, os.path.basename(in_daophot))
        temp_allstar  = os.path.join(temp_dir, os.path.basename(in_allstar))
        temp_out_als  = os.path.join(temp_dir, f"{out_als_filename}.als")
        temp_out_fits = os.path.join(temp_dir, f"{out_fits_filename}.fits")
        temp_log      = os.path.join(temp_dir, "allstar.log")
        out_log       = os.path.join(os.path.dirname(out_als), "allstar.log")

        shutil.copy(in_fits, temp_fits)
        shutil.copy(in_psf, temp_psf)
        shutil.copy(in_ap, temp_ap)
        shutil.copy(in_daophot, temp_daophot)
        shutil.copy(in_allstar, temp_allstar)
        
        temp_psf_filename = os.path.basename(in_psf)
        temp_ap_filename = os.path.basename(in_ap)

        if verbose:
            print(f"allstar: {filename}")
        check_file(temp_allstar, "opt file input: ")
        check_file(temp_fits, "fits file input: ")
        check_file(temp_psf, "psf file input")
        check_file(temp_ap, "positions list file input")
        overwrite_als = [""] if os.path.isfile(temp_out_als) else []
        do_RE = [""] if RE else ["RE=0\n"]
        cmd_list = ['allstar << EOF >> allstar.log',
                    *do_RE,
                    filename, temp_psf_filename, temp_ap_filename, 
                    out_als_filename, *overwrite_als, out_fits_filename, 
                    'exit', "EOF"]
        cmd = '\n'.join(cmd_list)
       
        # Ejecutar en la carpeta temporal
        if use_docker:
            runner = docker_run(use_docker)
            joined_cmds = '\n'.join(cmd_list[1:])
            cmd = "sh -c 'printf \"%s\\n\" \""+f"{joined_cmds}"+"\" | allstar >> allstar.log'"
        else:
            runner = run_proc
            cmd = cmd
        runner(cmd, temp_dir)

        #     runner(cmd, os.path.relpath(temp_dir, start=working_dir))
        # else:
        #     runner = run_proc
        #     cmd = cmd
        #     runner(cmd, temp_dir)

        # Mover el archivo de salida a la ubicación final
        final_out_als = move_file_noreplace(temp_out_als, out_als)
        final_out_fits = move_file_noreplace(temp_out_fits, out_fits)
        move_file_noreplace(temp_log, out_log)

        check_file(f"{final_out_als}", "als not created: ")
        check_file(f"{final_out_fits}", "sub fits not created: ")
        if verbose:
            print(f"  -> {final_out_als}")
            print(f"  -> {final_out_fits}")

    finally:
        # Limpiar carpeta temporal
        shutil.rmtree(temp_dir)
        pass

    return final_out_als, final_out_fits


from ....misc_tools import daophot_pbar
from shiny import module, reactive, render, ui
from faicons import icon_svg  # Para iconos en botones

@module.ui
def nav_panel_SUB_ui():
    return ui.page_fillable(
        ui.div(
            ui.input_select("fits_select", "Substract targets on FITS", choices={}, width="auto"),  # Lista FITS
            ui.input_select("table_psf_select", "Select PSF model", choices={}, width="auto"),  # Lista Tables
            ui.input_select("table_nei_select", "Select target list", choices={}, width="auto"),  # Lista Tables
            ui.input_select("table_lst_select", "Select exception list", choices={}, width="auto"),  # Lista Tables
            ui.input_action_button("sub_btn", "SUBSTRACT", icon=icon_svg("square-minus"), width="auto"),  # Botón compacto
            style="padding-right: 20px; border-right: 1px solid #dee2e6;"
        ),
    )

@module.server
def nav_panel_SUB_server(input, output, session, photfun_client, nav_table_sideview_update, input_tabs_main, input_tabs_daophot, navselected_fits):

    def update_select():
        fits_choices = {str(obj.id): f"[{obj.id}] {obj.alias}" for obj in photfun_client.fits_files}
        ui.update_select("fits_select", choices=fits_choices, selected=str(navselected_fits().id) if navselected_fits() else None)
        table_psf_choices = {str(obj.id): f"[{obj.id}] {obj.alias}" for obj in photfun_client.psf_files}
        ui.update_select("table_psf_select", choices=table_psf_choices)
        table_nei_choices = {str(obj.id): f"[{obj.id}] {obj.alias}" for obj in photfun_client.tables}
        ui.update_select("table_nei_select", choices=table_nei_choices)
        table_lst_choices = {"":"No exception"}
        table_lst_choices.update({str(obj.id): f"[{obj.id}] {obj.alias}" for obj in photfun_client.tables})
        ui.update_select("table_lst_select", choices=table_lst_choices)

    # Cargar opciones de FITS en el select_input
    @reactive.Effect
    @reactive.event(input_tabs_main)
    def _():
        if input_tabs_main()=="DAOPHOT":
            update_select()
    
    # Cargar opciones de FITS en el select_input
    @reactive.Effect
    @reactive.event(input_tabs_daophot)
    def _():
        if input_tabs_daophot()=="SUB":
            update_select()

    # Obtener el FITS seleccionado
    @reactive.Calc
    def selected_fits():
        selected_id = input.fits_select()
        return next((f for f in photfun_client.fits_files if str(f.id) == selected_id), None)

    # Obtener el Table seleccionado
    @reactive.Calc
    def selected_psf_table():
        selected_id = input.table_psf_select()
        return next((t for t in photfun_client.psf_files if str(t.id) == selected_id), None)

    # Obtener el Table seleccionado
    @reactive.Calc
    def selected_nei_table():
        selected_id = input.table_nei_select()
        return next((t for t in photfun_client.tables if str(t.id) == selected_id), None)

        # Obtener el Table seleccionado
    @reactive.Calc
    def selected_lst_table():
        selected_id = input.table_lst_select()
        return next((t for t in photfun_client.tables if str(t.id) == selected_id), False)

    # Ejecutar SUB al presionar el botón
    @reactive.Effect
    @reactive.event(input.sub_btn)
    def sub_action():
        fits_obj = selected_fits()
        psf_obj = selected_psf_table()
        nei_obj = selected_nei_table()
        lst_obj = selected_lst_table()
        if fits_obj and psf_obj and nei_obj:
            try:
                with ui.Progress(min=0, max=1) as p:
                    pbar = daophot_pbar(p, "Subtract")
                    out_fits_obj = photfun_client.sub(fits_obj.id, psf_obj.id, nei_obj.id, 
                                            lst_obj.id if lst_obj else False, pbar=pbar)
                ui.notification_show(f"Substracted fits\n -> [{out_fits_obj.id}] {out_fits_obj.alias}")
            except Exception as e:
                ui.notification_show(f"Error: {str(e)}", type="error")
        else:
            ui.notification_show("Error: FITS not selected.", type="warning")
        
        nav_table_sideview_update(tables=False, psf=False)
        update_select()

    return



import time
from datetime import timedelta

def daophot_pbar(pbar=None, func_msg="Processing", *args, **kwargs):
    """
    This function is intended to transform a progress_bar of shiny
    in something similar as tqdm with the following command line
    
    with ui.Progress(min=0, max=max_n) as p:
        pbar = daophot_pbar(p, "message")
        for var in pbar(foo_list):
            var
    """
    start_time = time.time()
    def yielder(iterable):
        # Modo iterable directo
        counter = 0
        total = len(iterable)
        amount = 1/total
        pbar.set(message=f"Executing {func_msg}", detail="Starting...")
        
        for item in iterable:
            yield item
            counter += 1
            
            elapsed_time = time.time() - start_time
            time_per_iter = elapsed_time / counter
            remaining_time = time_per_iter * (total - counter)
            
            remaining_time_str = (str(timedelta(seconds=int(remaining_time))) 
                                if counter > 1 else "Estimating...")
            
            pbar.inc(amount,
                    message=f"{func_msg}",
                    detail=f"Progress: {counter}/{total} | Time left: {remaining_time_str}")
    return yielder


from .input_local_file import input_local_file_ui, input_local_file_server
from .output_save_location import output_save_location_ui, output_save_location_server
from .samp_client import SAMPclient
from .nav_table_sideview import nav_table_sideview_ui, nav_table_sideview_server
from .nav_panel_IMAGE import nav_panel_IMAGE_ui, nav_panel_IMAGE_server
from .nav_panel_TABLE import nav_panel_TABLE_ui, nav_panel_TABLE_server
from .nav_panel_DAOPHOT import nav_panel_DAOPHOT_ui, nav_panel_DAOPHOT_server
from .nav_panel_SELECTION import nav_panel_SELECTION_ui, nav_panel_SELECTION_server
from .nav_panel_PHOTCUBE import nav_panel_PHOTCUBE_ui, nav_panel_PHOTCUBE_server
from .nav_panel_EXPORT import nav_panel_EXPORT_ui, nav_panel_EXPORT_server
from .nav_panel_LOGS import nav_panel_LOGS_ui, nav_panel_LOGS_server



import os
from subprocess import Popen, PIPE, STDOUT
from pathlib import Path
import threading
import psutil
from collections import deque
import uuid
try:
    import docker
    HAS_DOCKER = True
except ImportError:
    HAS_DOCKER = False


def run_proc(cmd, workdir, max_repeats=20, verbose=False):
    process = Popen(
        cmd, shell=True, cwd=workdir,
        stdout=PIPE, stderr=STDOUT, text=True
    )

    lines = deque(maxlen=max_repeats)
    killed_due_to_repeats = False

    def monitor():
        nonlocal killed_due_to_repeats
        for line in iter(process.stdout.readline, ''):
             # Muestra en consola
            lines.append(line.strip())
            if len(lines) == max_repeats and len(set(lines)) == 1:
                process.kill()
                killed_due_to_repeats = True
                break

    monitor_thread = threading.Thread(target=monitor)
    monitor_thread.start()
    process.wait()
    monitor_thread.join()

    if killed_due_to_repeats:
        raise RuntimeError(f"DAOPHOT error loop detected (exit code 137)")

    if process.returncode != 0:
        raise RuntimeError(f"DAOPHOT error (exit code {process.returncode})")

    return process.returncode


def init_docker(working_dir, n_proc=1, prev=[False], mem_fraction=0.4, force_docker=False):
    n_proc = os.cpu_count() if n_proc==-1 else n_proc
    if prev[0]:
        docker_stop_async(prev)
    
    if not force_docker:
        # Prefer local DAOPHOT if available
        try:
            run_proc("daophot << EOF\nexit\nEOF", workdir=working_dir, verbose=False)
            print("[PhotFun] Native DAOPHOT detected. Docker will not be used.")
            return [False]
        except Exception as e:
            print(f"[PhotFun] Native DAOPHOT not available:\n -> {e}")
            print("[PhotFun] Attempting to start Docker containers...")

    if not HAS_DOCKER:
        print(f"[PhotFun] Docker not available. Running locally.\n -> Import error")
        return [False]

    container_names = []
    try:
         # Calcula memoria total y mem_limit por contenedor
        total_mem = psutil.virtual_memory().total
        usable = total_mem * mem_fraction
        per_container = usable / n_proc
        # convierte a gigabytes redondeados
        per_gb = per_container / (1024**3)
        mem_str = f"{per_gb:.1f}g"
        
        docker_client = docker.from_env()
        docker_client.ping()  # Verifica si el daemon responde
        image_name = "ciquezada/photfun-daophot_wrapper"
        images = docker_client.images.list(name=image_name)

        # Pull de imagen si no existe
        if not images:
            print(f"[PhotFun] Downloading docker image '{image_name}'...")
            docker_client.images.pull(image_name)

        for i in range(n_proc):
            container = docker_client.containers.run(
                image=image_name,
                name = f"photfun_{uuid.uuid4().hex[:12]}",
                command="/bin/bash",
                volumes={str(Path(working_dir).resolve()): {
                    'bind': "/workdir", 
                    'mode': 'rw'
                }},
                working_dir="/workdir",
                tty=True,
                detach=True,
                mem_limit=mem_str,
                memswap_limit=mem_str,
            )
            container_names.append(container.name)
        print("[PhotFun] Docker DAOPHOT available.")
        print(f"[Docker] {mem_str} RAM per container available ")

    except Exception as e:
        print(f"[PhotFun] Docker not available. Running locally.\n -> {e}")
        return [False]

    return container_names

def docker_run(container_name):
    def docker_runner(cmd, workdir, verbose=False):
        # Ejecutar contenedor
        docker_client = docker.from_env()
        container = docker_client.containers.get(container_name)
        # if container.status != 'running':
        #     container.start()

        exec_res = container.exec_run(cmd=cmd, workdir=f"/workdir/{Path(workdir).as_posix()}")
    
        # stdout, stderr = exec_res.output
        # if stdout:
        #     print("[STDOUT]\n", stdout.decode())
        # if stderr:
        #     print("[STDERR]\n", stderr.decode())
        if exec_res.exit_code == 137:
            raise MemoryError("DAOPHOT error OOM-killed (exit code 137)")
        elif exec_res.exit_code != 0:
            raise RuntimeError(f"DAOPHOT error (exit code {exec_res.exit_code})")

    return docker_runner

def _stop_and_remove(name):
    if not name:
        print(f"[PhotFun] Docker is disabled.")
        return 0
    try:
        client = docker.from_env()
        container = client.containers.get(name)
        container.stop()
        container.remove()
        print(f"[Docker] Container '{name}' stopped and removed.")
    except Exception as e:
        print(f"[Docker] Error stopping/removing container '{name}': {e}. Manual intervention required.")

def docker_stop_async(container_names):
    """
    Para cada contenedor en container_names, lanza un hilo que
    haga stop() y remove(), pero NO espera a que terminen.
    """
    threads = []
    for name in container_names:
        t = threading.Thread(target=_stop_and_remove, args=(name,), daemon=True)
        t.start()
        threads.append(t)

    # Espera a que cada hilo termine
    for t in threads:
        t.join()

# def init_docker():
#     if not HAS_DOCKER:
#         print(f"[PhotFun] Docker not available. Running locally.\n -> Import error")
#         return False
#     try:
#         docker_client = docker.from_env()
#         docker_client.ping()  # Verifica si el daemon responde
#         image_name = "ciquezada/photfun-daophot_wrapper"
#         images = docker_client.images.list(name=image_name)

#         # Pull de imagen si no existe
#         if not len(images)>0:
#             print(f"[PhotFun] Downloading docker image '{image_name}'...")
#             docker_client.images.pull(image_name)

#         print("[PhotFun] Docker DAOPHOT available.")

#     except Exception as e:
#         print(f"[PhotFun] Docker not available. Running locally.\n -> {e}")
#         return False
#     return True

# def docker_run(cmd, workdir):
#     # Ejecutar contenedor
#     docker_client = docker.from_env()
#     container = docker_client.containers.run(
#         image="ciquezada/photfun-daophot_wrapper",
#         command="/bin/bash",
#         volumes={str(Path(workdir).resolve()): {
#                     'bind': "/workdir", 
#                     'mode': 'rw'
#                 }},
#         working_dir="/workdir",
#         tty=True,
#         detach=True
#     )
#     try:
#         exec_res = container.exec_run(cmd=cmd)
#         if exec_res.exit_code != 0:
#             raise RuntimeError(f"DAOPHOT error:\n{exec_res.exit_code}")
#     finally:
#         container.stop()
#         container.remove()


from shiny import reactive, render
from shiny.types import FileInfo
from shiny import App, ui
from pathlib import Path
import os
import time
import pandas as pd
import tempfile
from astropy.io.votable import from_table, writeto
from astropy.table import Table as astroTable
from urllib.parse import urlparse
from ..misc_tools import move_file_noreplace
from ..photfun_classes import PhotFun
from .gui_custom import (nav_table_sideview_server, nav_panel_IMAGE_server, 
                                            nav_panel_TABLE_server, nav_panel_DAOPHOT_server, 
                                            nav_panel_SELECTION_server, nav_panel_PHOTCUBE_server,
                                            nav_panel_EXPORT_server, nav_panel_LOGS_server, SAMPclient)

app_dir = Path(__file__).parent

def server(input, output, session):
    photfun_client = PhotFun()

    samp_client = SAMPclient()
    samp_message = reactive.Value({})

    # Reactivos de modulos
    nav_table_sideview_update, fits_df, tables_df, psf_df = nav_table_sideview_server("nav_table_sideview", photfun_client, samp_client)
    _ = nav_panel_IMAGE_server("nav_panel_IMAGE", photfun_client, samp_client, nav_table_sideview_update, fits_df)
    _ = nav_panel_TABLE_server("nav_panel_TABLE", photfun_client, samp_client, nav_table_sideview_update, tables_df)
    _ = nav_panel_DAOPHOT_server("nav_panel_DAOPHOT", photfun_client, nav_table_sideview_update, fits_df, tables_df, input.tabs_main)
    _ = nav_panel_SELECTION_server("nav_panel_SELECTION", photfun_client, nav_table_sideview_update, fits_df, tables_df)
    _ = nav_panel_PHOTCUBE_server("nav_panel_PHOTCUBE", photfun_client, samp_client, nav_table_sideview_update, input.tabs_main)
    _ = nav_panel_EXPORT_server("nav_panel_EXPORT", photfun_client, input.tabs_main)
    _ = nav_panel_LOGS_server("nav_panel_LOGS", photfun_client)

    nav_table_sideview_update()

    # Mantener una referencia al cliente para evitar garbage collection
    @reactive.Effect
    def init_samp():
        samp_client.start_samp()
        return samp_client

    # Verificador reactivo en el contexto principal
    @reactive.Effect
    def check_messages():
        reactive.invalidate_later(0.1)
        if samp_client.samp_receiver and samp_client.samp_receiver.new_message:
            new_params = samp_client.samp_receiver.params.copy()
            ui.notification_show(
                f"Received: {new_params.get('name', 'not named')}",
                duration=8,
            )
            samp_message.set(new_params)
            samp_client.samp_receiver.reset_flag()

    @reactive.Effect
    @reactive.event(samp_message)
    def samp_display():
        new_params = samp_message.get()
        # Parsear URL y obtener ruta local
        if new_params.get('mtype')!='table.load.votable' and new_params.get('mtype')!='image.load.fits':
            return
        
        url = urlparse(new_params['url'])
        with tempfile.NamedTemporaryFile(suffix=".vot", delete=False) as tmpfile:
            # Convertir DataFrame de pandas a Table de astropy
            astropy_table = astroTable.read(new_params['url'])
            votable = from_table(astropy_table)
            votable.description = new_params['name']
            
            # Escribir archivo temporal
            writeto(votable, tmpfile.name)   
            src_path = os.path.abspath(tmpfile.name) 
        
        # Construir ruta destino
        original_ext = os.path.splitext(src_path)[1]
        dest_path = os.path.join(photfun_client.working_dir, 
                        f"{os.path.basename(new_params['name'].replace(' ', '_'))}{original_ext}")
        
        # Copiar archivo
        final_dest = move_file_noreplace(src_path, dest_path)
        try:
            # Determinar tipo de archivo
            if new_params.get('mtype') == 'table.load.votable':
                print(f"PhotFun: import({new_params['name']})")
                photfun_client.add_table(final_dest)
                nav_table_sideview_update(fits=False, psf=False)
                
            elif new_params.get('mtype') == 'image.load.fits':
                print(f"PhotFun: import({new_params['name']})")
                photfun_client.add_fits(final_dest)
                nav_table_sideview_update(table=False, psf=False)
        except ValueError as e:
            ui.notification_show(str(e), type="error", duration=10)

    # Ejecutar cleanup cuando la sesión termine
    @session.on_ended
    def on_ended():
        photfun_client.clean_up()
        samp_client.stop_samp()
        pass




import os
import glob
from astropy.io import fits
from .phot_file import PhotFile


class PhotFits(PhotFile):
    def __init__(self, path, *args, **kwargs):
        super().__init__(path, *args, **kwargs)  # Hereda la lógica de PhotFile
        self._image_cache = {}  # Diccionario para almacenar imágenes cargadas

    def image(self, index=0):
        """Carga y devuelve la imagen FITS del archivo en la posición `index`."""
        if index < 0 or index >= len(self.path):
            raise IndexError(f"Index {index} is out of range. Available files: 0 to {len(self.path) - 1}.")
        
        if index not in self._image_cache:  # Solo carga si no está en caché
            self._image_cache[index] = fits.open(self.path[index])[0]

        return self._image_cache[index]

    def file_info(self, indx=0):
        """Devuelve información básica del archivo en un diccionario."""
        info = {
            "Filename": os.path.basename(self.path[indx]),
            "File type": self.file_type,
            "File location": os.path.dirname(self.path[indx]),
        }

        try:
            with fits.open(self.path[indx]) as hdul:
                header = hdul[0].header  # Extrae el header de la extensión primaria
                info["Header"] = {key: header[key] for key in header.keys()}  # Convierte el header en diccionario
        except Exception as e:
            info["Error"] = str(e)

        return info





from ....misc_tools import daophot_pbar
from shiny import module, reactive, render, ui
from faicons import icon_svg  # Para iconos en botones

@module.ui
def nav_panel_PICK_ui():
    return ui.page_fillable(
        ui.layout_columns(
            # Columna izquierda - Controles principales
            ui.div(
                ui.input_select("fits_select", "Pick best targets on FITS", choices={}, width="auto"),
                ui.input_select("table_select", "Select Targets", choices={}, width="auto"),
                ui.input_action_button("pick_btn", "PICK", icon=icon_svg("ranking-star"), width="auto"),
                style="padding-right: 20px; border-right: 1px solid #dee2e6;"
            ),
            # Columna derecha - Parámetros DAOPHOT específicos de PICK
            ui.div(
                ui.tooltip(
                    ui.input_numeric("param_fi", "Fitting Radius (px)", 
                                   value=4,  # Valor por defecto de 'fi' en opt_daophot_dict
                                   step=0.1
                                   ),
                    "Same or less than FWHM if the field is crowded"
                ),
                ui.tooltip(
                    ui.input_numeric("param_ps", "PSF Radius (px)", 
                                   value=20,  # Valor por defecto de 'ps' en opt_daophot_dict
                                   step=0.1,
                                   ),
                    "3 or 4 times the FWHM"
                ),
                ui.tooltip(
                    ui.input_numeric("param_stars", "Max stars", 
                                   value=200,  # Valor por defecto de 'fi' en opt_daophot_dict
                                   step=1
                                   ),
                    "100 stars or more for a PSF modeling is correct"
                ),
                ui.tooltip(
                    ui.input_numeric("param_minmag", "Min magnitude", 
                                   value=20,  # Valor por defecto de 'ps' en opt_daophot_dict
                                   step=0.1,
                                   ),
                    "Often weak stars are worse"
                ),
                style="padding-left: 20px;"
            ),
            col_widths=(8, 4)
        )
    )

@module.server
def nav_panel_PICK_server(input, output, session, photfun_client, nav_table_sideview_update, input_tabs_main, input_tabs_daophot, navselected_fits):

    def update_select():
        fits_choices = {str(obj.id): f"[{obj.id}] {obj.alias}" for obj in photfun_client.fits_files}
        ui.update_select("fits_select", choices=fits_choices, selected=str(navselected_fits().id) if navselected_fits() else None)
        table_choices = {str(obj.id): f"[{obj.id}] {obj.alias}" for obj in photfun_client.tables}
        ui.update_select("table_select", choices=table_choices)

    # Cargar opciones de FITS en el select_input
    @reactive.Effect
    @reactive.event(input_tabs_main)
    def _():
        if input_tabs_main()=="DAOPHOT":
            update_select()
            update_settings()
    
        # Cargar opciones de FITS en el select_input
    @reactive.Effect
    @reactive.event(input_tabs_daophot)
    def _():
        if input_tabs_daophot()=="PICK":
            update_select()
            update_settings()
            

    # Obtener el FITS seleccionado
    @reactive.Calc
    def selected_fits():
        selected_id = input.fits_select()
        return next((f for f in photfun_client.fits_files if str(f.id) == selected_id), None)

    # Obtener el Table seleccionado
    @reactive.Calc
    def selected_table():
        selected_id = input.table_select()
        return next((t for t in photfun_client.tables if str(t.id) == selected_id), None)

    # Modificamos los opt files
    def update_settings():
        # Sincronizar valores iniciales
        ui.update_numeric("param_fi", value=photfun_client.daophot_opt['fi'])
        ui.update_numeric("param_ps", value=photfun_client.daophot_opt['ps'])
        ui.update_numeric("param_stars", value=photfun_client.pick_max_stars)
        ui.update_numeric("param_minmag", value=photfun_client.pick_min_mag)

    @reactive.Effect
    @reactive.event(input.param_fi)
    def _():
        photfun_client.daophot_opt['fi'] = float(input.param_fi())
        
    @reactive.Effect
    @reactive.event(input.param_ps)
    def _():
        photfun_client.daophot_opt['ps'] = float(input.param_ps())

    @reactive.Effect
    @reactive.event(input.param_stars)
    def _():
        photfun_client.pick_max_stars = int(input.param_stars())
        
    @reactive.Effect
    @reactive.event(input.param_minmag)
    def _():
        photfun_client.pick_min_mag = int(input.param_minmag())

    # Ejecutar PICK al presionar el botón
    @reactive.Effect
    @reactive.event(input.pick_btn)
    def pick_action():
        fits_obj = selected_fits()
        table_obj = selected_table()
        if fits_obj and table_obj:
            try:
                with ui.Progress(min=0, max=1) as p:
                    pbar = daophot_pbar(p, "Pick")
                    out_table_obj = photfun_client.pick(fits_obj.id, table_obj.id, pbar=pbar)
                ui.notification_show(f"Selected {out_table_obj.df(0).shape[0]} sources\n -> [{out_table_obj.id}] {out_table_obj.alias}")
            except Exception as e:
                ui.notification_show(f"Error: {str(e)}", type="error")
        else:
            ui.notification_show("Error: FITS not selected.", type="warning")
        
        nav_table_sideview_update(fits=False, psf=False)
        update_select()

    return




import os


def temp_mkdir(folder_name):
    # making needed directories
    gen = (x for x in range(1,99999))
    aux_folder_name = folder_name[:-1]
    while True:
        try:
            os.mkdir(folder_name)
            break
        except OSError as error:
            folder_name = f"{aux_folder_name}{next(gen)}"
    return folder_name


# main.py (antes de todo lo demás)
import sys, asyncio
if sys.platform.startswith("win"):
    asyncio.set_event_loop_policy(
        asyncio.WindowsSelectorEventLoopPolicy()
    )

from pathlib import Path
from shiny import App, ui
from .server import server
from .gui_custom import (nav_table_sideview_ui, nav_panel_IMAGE_ui, 
                                            nav_panel_TABLE_ui, nav_panel_DAOPHOT_ui, 
                                            nav_panel_SELECTION_ui, nav_panel_PHOTCUBE_ui,
                                            nav_panel_EXPORT_ui, nav_panel_LOGS_ui)
import socket


def get_local_ipv4():
    # Crea un socket UDP temporal
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
        try:
            # No necesita estar disponible, solo para resolver la IP local usada
            s.connect(("8.8.8.8", 80))
            ip = s.getsockname()[0]
        except Exception:
            ip = "127.0.0.1"  # fallback si falla
    return ip

app_dir = Path(__file__).parent

app_ui = ui.page_fillable(
    ui.tags.head(
        ui.tags.style("""
        	:root {
    			--bslib-sidebar-main-bg: #f8f8f8;
  			}
  
            .main-header {
                background: linear-gradient(rgba(0, 0, 0, 0.7), rgba(0, 0, 0, 0.7)),
                            url('https://images.unsplash.com/photo-1465101162946-4377e57745c3?auto=format&fit=crop&w=1920&q=80');
                background-size: cover;
                background-position: center 30%;
                padding: 2rem;
                border-bottom: 1px solid rgba(255, 255, 255, 0.1);
            }
            
            .glass-panel {
                background: rgba(255, 255, 255, 0.9) !important;
                backdrop-filter: blur(10px);
                border-radius: 8px !important;
                box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);
            }
            
            .sidebar-light {
                background-color: #ffffff !important;
                border-right: 1px solid #e0e0e0 !important;
                padding: 1rem;
            }
            
            .dataframe-container {
                background: white !important;
                color: #333 !important;
            }
        """)
    ),
    
    ui.div(
        ui.card(
            ui.card_header(
                ui.h1("PHOTFUN", class_="display-4 mb-0 text-light"),
                ui.h4("Astronomical Photometry Suite", class_="text-secondary mt-1 text-light"),
                class_="main-header"
            ),
            ui.page_sidebar(
                nav_table_sideview_ui("nav_table_sideview"),
                ui.div(
                    ui.navset_card_pill(
                        ui.nav_menu(
                            "Files",
                            ui.nav_panel("Image", nav_panel_IMAGE_ui("nav_panel_IMAGE"), value="IMAGE"),
                            ui.nav_panel("Table", nav_panel_TABLE_ui("nav_panel_TABLE"), value="TABLE"),
                        ),            
                        ui.nav_panel("DAOphot", nav_panel_DAOPHOT_ui("nav_panel_DAOPHOT"), value="DAOPHOT"),
                        ui.nav_panel("Target Selection", nav_panel_SELECTION_ui("nav_panel_SELECTION"), value="SELECTION"),
                        ui.nav_panel("PHOTcube", nav_panel_PHOTCUBE_ui("nav_panel_PHOTCUBE"), value="PHOTCUBE"),
                        ui.nav_panel("Export", nav_panel_EXPORT_ui("nav_panel_EXPORT"), value="EXPORT"),
                        ui.nav_panel("Logs", nav_panel_LOGS_ui("nav_panel_LOGS"), value="LOGS"),
                        id="tabs_main",
                    ),
                    class_="glass-panel"
                ),
                class_="mt-3",
                width="100%"
            ),
            class_="glass-panel border-0"
        ),
        style="padding: 2rem; width: 100%; height: 60vh;"
    ),
    fillable=True,
    fillable_mobile=True,
    fullscreen=True,
)
app = App(app_ui, server)

def run_photfun():
    print(f"INFO:     Lan in http://{get_local_ipv4()}:8000")
    print(f"INFO:     Localhost in http://localhost:8000 or http://127.0.0.1:8000")
    app.run(host="0.0.0.0", port=8000)

if __name__ == "__main__":
    app.run()


import os
from shiny import module, reactive, render, ui
from faicons import icon_svg
import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.colors import LogNorm
from pprint import pformat  # Import pprint para formateo de texto
from . import input_local_file_ui, input_local_file_server
from . import output_save_location_ui, output_save_location_server
from ...misc_tools import cube_slicer, spectra_compile, daophot_pbar

@module.ui
def nav_panel_PHOTCUBE_ui():
    return ui.page_fluid(
        ui.layout_column_wrap(
            [
                ui.h4("Slice & Load Datacube"),
                ui.div(
                    input_local_file_ui("load_slice_datacube", "Search DataCube", width="auto"),
                ),
            ],
            [
                ui.h4("Spectra compiler"),
                ui.div(
                    ui.input_select("master_select", "Select MasterList", choices={}, width="auto"),
                    ui.input_select("table_select", "Select Spectra Photometry", choices={}, width="auto"),
                    output_save_location_ui("btn_export", "Compile Spectra"),
                    style="padding-left: 20px; border-left: 1px solid #dee2e6;"
                ),
            ],
        ),
        
    )

@module.server
def nav_panel_PHOTCUBE_server(input, output, session, photfun_client, samp_client, 
                            nav_table_sideview_update, input_tabs_main):
    event_load_local_fits, input_load_local_fits  = input_local_file_server("load_slice_datacube", ".fits")
    event_save_location, input_save_location  = output_save_location_server("btn_export")

    # Cargar opciones de FITS en el select_input
    @reactive.Effect
    @reactive.event(input_tabs_main)
    def _():
        if input_tabs_main()=="PHOTCUBE":
            objects = [table for table in photfun_client.tables]
            choices = {obj.id: f"[{obj.id}] {obj.alias}" for obj in objects}
            ui.update_select("master_select", choices=choices)
            objects = [table for table in photfun_client.tables]
            choices = {obj.id: f"[{obj.id}] {obj.alias}" for obj in objects}
            ui.update_select("table_select", choices=choices)

    # Obtener el Table seleccionado
    @reactive.Calc
    def selected_master_table():
        selected_id = input.master_select()
        return next((t for t in photfun_client.tables if str(t.id) == selected_id), None)

    # Obtener el Table seleccionado
    @reactive.Calc
    def selected_spectra_table():
        selected_id = input.table_select()
        return next((t for t in photfun_client.tables if str(t.id) == selected_id), None)


    # Evento al guardar archivos FITS
    @reactive.Effect
    @reactive.event(event_save_location)
    def _():
        out_path = input_save_location()
        master_obj = selected_master_table()
        spectra_obj = selected_spectra_table()
        print("PhotFun: Compile spectra")

        if master_obj and spectra_obj:
            try:
                with ui.Progress(min=0, max=1) as p1:
                    p1.set(message="Compiling spectra")
                    pbar_extract = daophot_pbar(p1, "Compiling spectra")
                    with ui.Progress(min=0, max=1) as p2:
                        p2.set(message="Saving spectra", detail="Waiting compilation...")
                        pbar_save = daophot_pbar(p2, "Saving spectra")
                        spectra_compile(spectra_obj.path, master_obj.path[0], out_path, 
                                            pbar_extract=pbar_extract, pbar_save=pbar_save)
                ui.notification_show(f"Spectra compilation complete")
                print(" -> Spectra compilation complete")
            except Exception as e:
                ui.notification_show(f"Error: {str(e)}", type="error")
        else:
            ui.notification_show("Error: Master or Photometry not selected.", type="warning")

    # Evento al cargar archivos FITS
    @reactive.Effect
    @reactive.event(event_load_local_fits)
    def _():
        archivos = input_load_local_fits()
        if not archivos:
            return
        print("PhotFun: Slice")
        carpetas = [f for f in archivos if os.path.isdir(f)]
        archivos_fits = [f for f in archivos if os.path.isfile(f)]
        if len(carpetas) or len(archivos)>1:
            ui.notification_show("Error: not single FITS selected.", type="warning")
            return 0
        else:
            try:
                with ui.Progress(min=0, max=1) as p:
                    pbar = daophot_pbar(p, "Slicing Cube")
                    datacube_slices = cube_slicer(archivos_fits[0], pbar)
                out_fits_obj = photfun_client.add_fits(datacube_slices)
                ui.notification_show(f"Datacube slice complete\n -> [{out_fits_obj.id}] {out_fits_obj.alias}")
                nav_table_sideview_update(tables=False, psf=False)
            except Exception as e:
                ui.notification_show(f"Error: {str(e)}", type="error")
        # nav_table_sideview_update(tables=False, psf=False)

    return



import os
from shiny import module, reactive, render, ui

def get_dir_contents(path, ext_filter=None, filter_file=None):
    items = os.listdir(path)
    folders = sorted([".", ".."] + [f for f in items if os.path.isdir(os.path.join(path, f))], key=lambda f: f.lower())
    files = [f for f in items if os.path.isfile(os.path.join(path, f))]
    if ext_filter:
        if isinstance(ext_filter, str):  
            ext_filter = [ext_filter]
        files = [f for f in files if os.path.splitext(f)[1].lower() in ext_filter]
    if filter_file and filter_file!="":
        files = [f for f in files if f.lower().startswith(filter_file.lower())]
    files = sorted(["."] + files, key=lambda f: f.lower())
    return folders, files


ROOT_PATH = os.getcwd()


@module.ui
def input_local_file_ui(label="Load File", width="100%"):
    return ui.page_fluid(
        ui.input_action_button("button_open", label, width=width),
        # ui.tags.head(ui.tags.style("""
        #     .modal-dialog { 
        #         width: flex;
        #         height: flex;
        #     }
        #     .modal-content {
        #         height: flex;
        #     }
        #     #folder_modal .select-container {
        #         display: flex;
        #         gap: 15px;
        #     }
        #     #folder_modal .scrollable {
        #         height: 100%;
        #         overflow: auto;
        #     }
        # """))
    )

@module.server
def input_local_file_server(input, output, session, ext_filter):
    current_path = reactive.value(ROOT_PATH)
    selected_path_out = reactive.value(ROOT_PATH)
    
    @reactive.effect
    @reactive.event(input.button_open)
    def _():
        folders, files = get_dir_contents(current_path(), ext_filter)
        FOLDER_BROWSER = ui.modal(
            ui.div(
                ui.h4("Select a folder", class_="modal-title"),
                ui.output_text_verbatim("current_path_display"),
                ui.layout_column_wrap(
                    ui.div(
                        ui.input_select("in_folder", "Folders", 
                                      choices=folders, 
                                      selected=".", 
                                      size=12,
                                      ),
                        # class_="col-6 p-0",
                        # style="display: flex; justify-content: flex-start;",
                        
                    ),
                    ui.div(
                        ui.input_select("in_file", "Files", 
                                      choices=files, 
                                      selected=".", 
                                      size=12,
                                      multiple=True,
                                      ),
                        ui.input_text("filter_file", None, placeholder="Type to filter..."), 
                        ui.input_action_button("button_select_file", 
                                              "Select File",
                                              class_="btn-primary"),
                        # class_="col-6 d-flex flex-column",
                		# style="display: flex; justify-content: flex-start;",
                        
                    ),
                    # class_="select-container",
                	# style="margin-top: 20px; display: flex; gap: 10px; justify-content: flex-start;"
                ),
                # class_="d-flex flex-column h-80",
            ),
            id="folder_modal",
            size="xl",
            easy_close=True,
            footer=None,
            # style="justify-content: width: 800px"
            # class_="modal-dialog-centered"
        )
        ui.modal_show(FOLDER_BROWSER)
    
    @reactive.effect
    @reactive.event(input.in_folder)
    def change_directory():
        selected = input.in_folder()
        new_path = os.path.abspath(os.path.join(current_path(), selected))
        if os.path.isdir(new_path):
            current_path.set(new_path)
            folders, files = get_dir_contents(current_path(), ext_filter, input.filter_file())
            ui.update_select("in_folder", choices=folders)
            ui.update_select("in_file", choices=files)
    
    @reactive.effect
    @reactive.event(input.filter_file)
    def filtering_files():
        folders, files = get_dir_contents(current_path(), ext_filter, input.filter_file())
        ui.update_select("in_folder", choices=folders)
        ui.update_select("in_file", choices=files)


    @output
    @render.text
    def current_path_display():
        return current_path()
        
    @reactive.effect
    @reactive.event(input.button_select_file)
    def select_file():
        selected_path = input.in_file()
        selected_path_out.set([os.path.abspath(os.path.join(current_path(), p)) for p in selected_path])
        
        
    return input.button_select_file, selected_path_out

# app = App(app_ui, server)

# if __name__ == "__main__":
#     app.run()



from ....misc_tools import daophot_pbar
from shiny import module, reactive, render, ui
from faicons import icon_svg  # Para iconos en botones

    

@module.ui
def nav_panel_FIND_ui():
    return ui.page_fillable(
        ui.layout_columns(
            # Columna izquierda - Controles principales
            ui.div(
                ui.input_select("fits_select", "Find targets on FITS", choices={}, width="auto", selected=None),
                ui.input_action_button("find_btn", "FIND", icon=icon_svg("magnifying-glass"), width="auto"),
                style="padding-right: 20px; border-right: 1px solid #dee2e6;"
            ),
            # Columna derecha - Parámetros DAOPHOT
            ui.div(
                ui.tooltip(
                    ui.input_numeric("param_fw", "FWHM (pixels)", value=3, step=0.1),
                    "Full Width at Half Maximum of stars in pixels"
                ),
                ui.tooltip(
                    ui.input_numeric("param_th", "Threshold (σ)", value=5, step=0.1),
                    "Sigma threshold for a detection"
                ),
                                ui.tooltip(
                ui.input_numeric("param_sum", "Summed", 
                                   value=1,  # Valor por defecto de 'fi' en opt_daophot_dict
                                   step=1
                                   ),
                    "Sum of several individual exposures, how many?"
                ),
                ui.tooltip(
                    ui.input_numeric("param_average", "Averaged", 
                                   value=1,  # Valor por defecto de 'ps' en opt_daophot_dict
                                   step=1,
                                   ),
                    "Average of several individual exposures, how many?"
                ),
                style="padding-left: 20px;"
            ),
            col_widths=(8, 4)
        )
    )

@module.server
def nav_panel_FIND_server(input, output, session, photfun_client, nav_table_sideview_update, input_tabs_main, input_tabs_daophot, navselected_fits):

    def update_select():
        fits_choices = {str(obj.id): f"[{obj.id}] {obj.alias}" for obj in photfun_client.fits_files}
        ui.update_select("fits_select", choices=fits_choices, selected=str(navselected_fits().id) if navselected_fits() else None)

    # Cargar opciones de FITS en el select_input
    @reactive.Effect
    @reactive.event(input_tabs_main)
    def _():
        if input_tabs_main()=="DAOPHOT":
            update_select()
            update_settings()
    
        # Cargar opciones de FITS en el select_input
    @reactive.Effect
    @reactive.event(input_tabs_daophot)
    def _():
        if input_tabs_daophot()=="FIND":
            update_select()
            update_settings()

    # Obtener el FITS seleccionado
    @reactive.Calc
    def selected_fits():
        selected_id = input.fits_select()
        return next((f for f in photfun_client.fits_files if str(f.id) == selected_id), None)
    
    # Modificamos los opt files
    def update_settings():
        # Sincronizar valores iniciales
        ui.update_numeric("param_fw", value=photfun_client.daophot_opt['fw'])
        ui.update_numeric("param_th", value=photfun_client.daophot_opt['th'])
        ui.update_numeric("param_sum", value=photfun_client.find_sum)
        ui.update_numeric("param_average", value=photfun_client.find_average)

    @reactive.Effect
    @reactive.event(input.param_fw)
    def _():
        photfun_client.daophot_opt['fw'] = float(input.param_fw())
        
    @reactive.Effect
    @reactive.event(input.param_th)
    def _():
        photfun_client.daophot_opt['th'] = float(input.param_th())

    @reactive.Effect
    @reactive.event(input.param_sum)
    def _():
        photfun_client.find_sum = int(input.param_sum())
        
    @reactive.Effect
    @reactive.event(input.param_average)
    def _():
        photfun_client.find_average = int(input.param_average())        

    # Ejecutar Find al presionar el botón
    @reactive.Effect
    @reactive.event(input.find_btn)
    def find_action():
        fits_obj = selected_fits()
        if fits_obj:
            try:
                with ui.Progress(min=0, max=1) as p:
                    pbar = daophot_pbar(p, "Find")
                    out_table_obj = photfun_client.find(fits_obj.id, pbar=pbar)
                ui.notification_show(
                    f"Found {out_table_obj.df(0).shape[0]} sources"
                    f" -> [{out_table_obj.id}] {out_table_obj.alias}"
                )
            except Exception as e:
                ui.notification_show(f"Error: {str(e)}", type="error")
        else:
            ui.notification_show("Error: FITS not selected.", type="warning")
        
        nav_table_sideview_update(fits=False, psf=False)
        update_select()

    return



import os
import re
import warnings
import argparse
import numpy as np
import pandas as pd
from tqdm import tqdm
import nest_asyncio
import asyncio


def mag2flux(mag):
    return 10**(-2/5 * (mag-25))

def read_als(filepath):
    col_names = ['ID', 'X', 'Y', "MAG", "merr", "msky", "niter", "chi", "sharpness"]
    try:
        return pd.read_csv(filepath, sep='\s+', skiprows=3, names=col_names, usecols=range(9), dtype={"ID": int})
    except FileNotFoundError:
        return None

def read_master(filepath):
    col_names = ['ID']
    try:
        return pd.read_csv(filepath, sep='\s+', skiprows=3, names=col_names, usecols=[0], dtype={"ID": int})
    except FileNotFoundError:
        return None

def parse_wavelength_from_name(filename):
    match = re.search(r"slice_(\d+p\d+)\.als", filename)
    if match:
        return float(match.group(1).replace('p', '.'))
    else:
        raise ValueError(f"No valid wavelength found in filename: {filename}")

def spectra_compile(als_dir, master_list, output_dir, pbar_extract=tqdm, pbar_save=tqdm):
    nest_asyncio.apply()
    os.makedirs(output_dir, exist_ok=True)
    star_ids = read_master(master_list)["ID"].astype(int).tolist()
    
    # Crear lista de paths de archivos .als
    if isinstance(als_dir, list):
        als_files = [
            f
            for f in als_dir
            if re.match(r".*_slice_\d+p\d+\.als", f)
        ]
    elif os.path.isdir(als_dir):
        als_files = [
            os.path.join(als_dir, f)
            for f in os.listdir(als_dir)
            if re.match(r".*_slice_\d+p\d+\.als", f)
        ]
    else:
        raise ValueError("Selected path is not a dir or even a list of .als")
    
    if not als_files:
        raise ValueError("No valid .als files found")

    # Extraer longitudes de onda y ordenar
    slice_info = sorted(
        [(parse_wavelength_from_name(os.path.basename(path)), path) for path in als_files],
        key=lambda x: x[0]
    )

    wavelengths = [w for w, _ in slice_info]
    spectra_data = {star_id: [np.nan] * len(wavelengths) for star_id in star_ids}
    errors_data = {star_id: [np.nan] * len(wavelengths) for star_id in star_ids}

    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    try:
        for idx, (wavelength, als_file) in pbar_extract([*enumerate(slice_info)]):
            als_data = read_als(als_file)
            if als_data is not None:
                als_data = als_data.set_index("ID")
                for star_id in star_ids:
                    if star_id in als_data.index:
                        spectra_data[star_id][idx] = als_data.loc[star_id, "MAG"]
                        errors_data[star_id][idx] = als_data.loc[star_id, "merr"]
    finally:
        loop.close()

    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    try:
        for star_id in pbar_save(star_ids):
            spectra_flux = mag2flux(np.array(spectra_data[star_id]))
            spectra_flux_sig = mag2flux(np.array(spectra_data[star_id]) - np.array(errors_data[star_id])) - spectra_flux
            df = pd.DataFrame({
                "wavelength": wavelengths,
                "flux": spectra_flux,
                "flux_sig": spectra_flux_sig
            })
            df.to_csv(os.path.join(output_dir, f"id{star_id}.csv"), index=False)
    finally:
        loop.close()
    return 0
    
if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Extract spectra from ALS files using encoded wavelengths in filenames")
    parser.add_argument("-a", "--als_dir", type=str, required=True, help="Directory containing .als files")
    parser.add_argument("-m", "--master_list", type=str, required=True, help="CSV file with star IDs")
    parser.add_argument("-o", "--output_dir", type=str, required=True, help="Output directory for spectra CSV files")

    args = parser.parse_args()
    spectra_compile(args.als_dir, args.master_list, args.output_dir)



import os
from urllib.parse import urlparse

def is_votable_url(url):
    parsed = urlparse(url)
    if not all([parsed.scheme in ['http', 'https'], parsed.netloc]):
        return False
    return parsed.path.lower()

class PhotFile:
    def __init__(self, path, alias=None, *args, **kwargs):
        self.path = None
        if isinstance(path, list):  # Lista explícita de archivos
            self.path = path
        elif os.path.isdir(path):  # Es un directorio, obtenemos todos los archivos
            self.path = sorted([
                os.path.join(path, f) for f in os.listdir(path)
                if os.path.isfile(os.path.join(path, f))
            ], key=lambda f: f.lower())
        elif os.path.isfile(path):  # Es un único archivo, lo convertimos en una lista
            self.path = [path]
        elif is_votable_url(path).endswith('.vot') or is_votable_url(path).endswith('.fits'): 
                self.path = [path]       
        else:
            raise ValueError(f"Invalid path: {path}")

        if not self.path or not len(self.path):
            raise ValueError(f"The directory '{path}' is empty or contains no valid files.")

        # Alias: Si hay varios archivos, usa el primero; si es un archivo único, usa su nombre.
        self.alias = f"[>] {os.path.basename(self.path[0])}" if len(self.path) > 1 else os.path.basename(self.path[0])
        if alias:
            self.alias = alias

        # Validación de extensiones
        extensions = {os.path.splitext(f)[1].lower() for f in self.path}
        if len(extensions) > 1:
            raise ValueError(f"Multiple file types detected: {extensions}. All files must have the same extension.")

        self.file_type = list(extensions)[0]

        self.header = None




#!/usr/bin/env python3

import os
import sys
import argparse
import warnings
import numpy as np
from tqdm import tqdm
from astropy.io import fits
from astropy.wcs import WCS
import nest_asyncio
import asyncio


def find_valid_hdu(fits_path):
    """Finds the first HDU in the FITS file that contains 3D data."""
    with fits.open(fits_path) as hdul:
        for i, hdu in enumerate(hdul):
            if hasattr(hdu, 'data') and hdu.data is not None and hdu.data.ndim == 3:
                return i  # Return the first valid HDU index
    raise ValueError(f"No valid 3D data found in {fits_path}")

def cube_slicer(fits_path, pbar=tqdm):
    nest_asyncio.apply()
    """Loads a 3D FITS cube, extracts each slice, and saves them as individual files."""

    # Check if the file exists
    if not os.path.exists(fits_path):
        print(f"Error: The file '{fits_path}' does not exist.")
        sys.exit(1)

    # Find the correct HDU with 3D data
    hdu_index = find_valid_hdu(fits_path)

    # Load the FITS cube
    with fits.open(fits_path) as hdul:
        data = hdul[hdu_index].data
        header = hdul[hdu_index].header
        # Get wavelength values from WCS
        wcs = WCS(header, naxis=0)
        num_slices = data.shape[0]
        coords = np.zeros((num_slices, 3))
        coords[:, 2] = np.arange(num_slices)
        wavelengths = wcs.all_pix2world(coords, 2)[:, 2]*1e10

    # Ensure the FITS file is a 3D cube
    if data.ndim != 3:
        print(f"Error: The file '{fits_path}' is not a 3D cube.")
        sys.exit(1)

    # Create output directory
    base_name = os.path.splitext(os.path.basename(fits_path))[0]
    output_dir = os.path.join(os.path.dirname(fits_path), f"{base_name}_slices")
    os.makedirs(output_dir, exist_ok=True)

    # num_slices = data.shape[0]  # Spectral axis is the first dimension

    # Save each slice as a separate FITS file using wavelength
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    try:
        for i in pbar(range(data.shape[0])):
            slice_data = data[i, :, :]
            slice_header = header.copy()
            slice_header["SLICE"] = i
            wavelength = wavelengths[i]
            wavelength_str = f"{wavelength:.5f}".replace('.', 'p')

            slice_filename = os.path.join(output_dir, f"{base_name}_slice_{wavelength_str}.fits")

            # Save the slice
            fits.writeto(slice_filename, slice_data, slice_header, overwrite=True)
    finally:
        loop.close()
    return output_dir

def main():
    parser = argparse.ArgumentParser(
        description="Splits a 3D FITS cube into individual slices without using SpectralCube."
    )
    parser.add_argument(
        "fits_path",
        type=str,
        help="Path to the FITS cube file."
    )

    args = parser.parse_args()
    cube_slicer(args.fits_path)

if __name__ == "__main__":
    main()



from ..misc_tools import check_file, temp_mkdir, move_file_noreplace
import os
import tempfile
import shutil
from .docker_handler import run_proc, docker_run


def find(in_fits, in_daophot, out_coo, sum_aver="1,1", verbose=True, 
                	use_docker=None, working_dir=".") -> [".coo"]: 
    try:
        # Copiar archivos necesarios a la carpeta temporal
        filename = os.path.splitext(os.path.basename(in_fits))[0]
        # Crear carpeta temporal
        temp_dir = os.path.abspath(temp_mkdir(os.path.join(working_dir, f"{filename}_FIND_0")))
        temp_fits = os.path.join(temp_dir, os.path.basename(in_fits))
        temp_opt  = os.path.join(temp_dir, os.path.basename(in_daophot))
        temp_coo  = os.path.join(temp_dir, os.path.basename(out_coo))
        temp_log  = os.path.join(temp_dir, "find.log")
        out_log = os.path.join(os.path.dirname(out_coo), "find.log")

        shutil.copy(in_fits, temp_fits)
        shutil.copy(in_daophot, temp_opt)
        
        out_coo_filename = os.path.basename(temp_coo)

        if verbose:
            print(f"daophot: find({filename})")
        check_file(temp_opt, "opt file input: ")
        check_file(temp_fits, "fits file input: ")
        overwrite = [""] if os.path.isfile(out_coo_filename) else []
        cmd_list = ['daophot << EOF >> find.log', f'at {filename}', 
                        'find', f'{sum_aver}', out_coo_filename,
                        *overwrite,
                        'y', 
                        'exit', 'EOF']
        cmd = '\n'.join(cmd_list)

        
        # Ejecutar en la carpeta temporal
        if use_docker:
            runner = docker_run(use_docker)
            joined_cmds = '\n'.join(cmd_list[1:])
            cmd = "sh -c 'printf \"%s\\n\" \""+f"{joined_cmds}"+"\" | daophot >> find.log'"
            runner(cmd, os.path.relpath(temp_dir, start=working_dir))
        else:
            runner = run_proc
            cmd = cmd
            runner(cmd, temp_dir)

        # Mover el archivo de salida a la ubicación final
        final_out_coo = move_file_noreplace(temp_coo, out_coo)
        move_file_noreplace(temp_log, out_log)

        check_file(final_out_coo, "coo not created: ")
        if verbose:
            print(f"  -> {final_out_coo}")

    finally:
        # Limpiar carpeta temporal
        shutil.rmtree(temp_dir)
        pass

    return final_out_coo


import os
from shiny import module, reactive, render, ui
from faicons import icon_svg
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.colors import LogNorm
# from astropy.visualization import ZScaleInterval
from .nav_panel_daophot_fn import (
    nav_panel_FIND_ui, nav_panel_FIND_server,
    nav_panel_PHOT_ui, nav_panel_PHOT_server,
    nav_panel_PICK_ui, nav_panel_PICK_server,
    nav_panel_PSF_ui, nav_panel_PSF_server,
    nav_panel_SUB_ui, nav_panel_SUB_server,
    nav_panel_ALLSTAR_ui, nav_panel_ALLSTAR_server,
    nav_panel_DAOMATCH_ui, nav_panel_DAOMATCH_server,
    nav_panel_CREATE_MASTER_ui, nav_panel_CREATE_MASTER_server,
    nav_panel_opt_DAOPHOT_ui, nav_panel_opt_DAOPHOT_server,
    nav_panel_opt_PHOTO_ui, nav_panel_opt_PHOTO_server,
    nav_panel_opt_ALLSTAR_ui, nav_panel_opt_ALLSTAR_server,
)


@module.ui
def nav_panel_DAOPHOT_ui():
    m = ui.page_fillable(
            ui.layout_columns(
                ui.page_fillable(
                    ui.div(
                        ui.h4("Loaded Data", class_="d-inline"),
                        ui.div(
                            ui.div(
                                ui.input_action_button(
                                    "toggle_docker",
                                    "Connect Docker",  # Puedes actualizar dinámicamente el label desde el server
                                    icon=icon_svg("plug"),
                                    class_="btn-sm btn-outline-primary"
                                ),
                                class_="d-flex flex-column align-items-end"
                            ),
                            class_="float-end"
                        ),
                        ui.output_plot("plot_fits"),
                    )
                ),
                ui.page_fillable(
                    ui.navset_card_tab(  
                        ui.nav_panel("FIND", nav_panel_FIND_ui("nav_panel_FIND"), value="FIND"),
                        ui.nav_panel("PHOT", nav_panel_PHOT_ui("nav_panel_PHOT"), value="PHOT"),
                        ui.nav_panel("PICK", nav_panel_PICK_ui("nav_panel_PICK"), value="PICK"),
                        ui.nav_panel("PSF", nav_panel_PSF_ui("nav_panel_PSF"), value="PSF"),
                        ui.nav_panel("SUBTRACT", nav_panel_SUB_ui("nav_panel_SUB"), value="SUB"),
                        ui.nav_panel("ALLSTAR", nav_panel_ALLSTAR_ui("nav_panel_ALLSTAR"), value="ALLSTAR"),
                        ui.nav_panel("DAOMATCH", nav_panel_DAOMATCH_ui("nav_panel_DAOMATCH"), value="DAOMATCH"),
                        ui.nav_panel("MASTER", nav_panel_CREATE_MASTER_ui("nav_panel_CREATE_MASTER"), value="CREATE_MASTER"),
                        ui.nav_menu(
                            "Settings",
                            ui.nav_panel("DAOPHOT", nav_panel_opt_DAOPHOT_ui("nav_panel_opt_DAOPHOT"), value="opt_DAOPHOT"),
                            ui.nav_panel("PHOTO", nav_panel_opt_PHOTO_ui("nav_panel_opt_PHOTO"), value="opt_PHOTO"),
                            ui.nav_panel("ALLSTAR", nav_panel_opt_ALLSTAR_ui("nav_panel_opt_ALLSTAR"), value="opt_ALLSTAR"),
                        ),
                        id="tabs_daophot",  
                    ),
                ),  
                col_widths=(6, 6),
            ),
        )
    return m


@module.server
def nav_panel_DAOPHOT_server(input, output, session, photfun_client, nav_table_sideview_update, fits_df, tables_df, input_tabs_main):
        
    @reactive.Effect
    @reactive.event(input_tabs_main)
    def _():
        if input_tabs_main()=="DAOPHOT":
            docker_toggle_handler()
    
    def docker_toggle_handler():
        if photfun_client.docker_container[0]:
            ui.update_action_button("toggle_docker", 
                label="Disconnect Docker",
                icon=icon_svg("plug-circle-xmark"), 
            )
        else:
            ui.update_action_button("toggle_docker", 
                label="Connect Docker",
                icon=icon_svg("plug"),
            )

    # Obtener la imagen FITS seleccionada
    @reactive.Calc
    def selected_fits():
        selected_row = fits_df.data_view(selected=True)
        if selected_row.empty:
            return None
        selected_id = selected_row.iloc[0]["FITS"]
        fits_obj = next((f for f in photfun_client.fits_files if f.id == selected_id), None)
        return fits_obj

    # Obtener la tabla seleccionada
    @reactive.Calc
    def selected_table():
        selected_row = tables_df.data_view(selected=True)
        if selected_row.empty:
            return None
        selected_id = selected_row.iloc[0]["Table"]
        table_obj = next((f for f in photfun_client.tables if f.id == selected_id), None)
        return table_obj

    _ = nav_panel_FIND_server("nav_panel_FIND", photfun_client, nav_table_sideview_update, input_tabs_main, input.tabs_daophot, selected_fits)
    _ = nav_panel_PHOT_server("nav_panel_PHOT", photfun_client, nav_table_sideview_update, input_tabs_main, input.tabs_daophot, selected_fits)
    _ = nav_panel_PICK_server("nav_panel_PICK", photfun_client, nav_table_sideview_update, input_tabs_main, input.tabs_daophot, selected_fits)
    _ = nav_panel_PSF_server("nav_panel_PSF", photfun_client, nav_table_sideview_update, input_tabs_main, input.tabs_daophot, selected_fits)
    _ = nav_panel_SUB_server("nav_panel_SUB", photfun_client, nav_table_sideview_update, input_tabs_main, input.tabs_daophot, selected_fits)
    _ = nav_panel_ALLSTAR_server("nav_panel_ALLSTAR", photfun_client, nav_table_sideview_update, input_tabs_main, input.tabs_daophot, selected_fits)
    _ = nav_panel_DAOMATCH_server("nav_panel_DAOMATCH", photfun_client, nav_table_sideview_update, input_tabs_main, input.tabs_daophot)
    _ = nav_panel_CREATE_MASTER_server("nav_panel_CREATE_MASTER", photfun_client, nav_table_sideview_update, input_tabs_main, input.tabs_daophot)
    _ = nav_panel_opt_DAOPHOT_server("nav_panel_opt_DAOPHOT", photfun_client, input_tabs_main, input.tabs_daophot)
    _ = nav_panel_opt_PHOTO_server("nav_panel_opt_PHOTO", photfun_client, input_tabs_main, input.tabs_daophot)
    _ = nav_panel_opt_ALLSTAR_server("nav_panel_opt_ALLSTAR", photfun_client, input_tabs_main, input.tabs_daophot)


    # Graficar la imagen FITS con posiciones de la tabla si está disponible
    @render.plot()
    def plot_fits():
        fits_obj = selected_fits()
        table_obj = selected_table()
        fits_image = fits_obj.image(0) if fits_obj else None
        table_df = table_obj.df(0) if table_obj else None
        
        if not fits_image:
            return
        
        fig, ax = plt.subplots(figsize=(7.5, 7.5))
        image_data = np.array(fits_image.data)
        image_data = np.nan_to_num(image_data, nan=0)
        image_data[image_data <= 0] = 0.0001
        # ax.imshow(image_data, cmap='gray', norm=LogNorm())
        vmin, vmax = np.percentile(image_data, [25, 90])
        ax.imshow(image_data, cmap='gray', norm=LogNorm(vmin=vmin, vmax=vmax))
        ax.invert_yaxis()
        
        if table_df is not None and "X" in table_df and "Y" in table_df:
            ax.scatter(table_df["X"], table_df["Y"], edgecolors='red', facecolors='none', s=30, alpha=0.3)

        
        fig.tight_layout()
        return fig
    
    @reactive.Effect
    @reactive.event(input.toggle_docker)
    def toggle_docker():
        if photfun_client.docker_container[0]:
            # Mensaje pequeño y gris mientras desconecta
            ui.notification_show(
                ui.HTML("Disconnecting <strong>Docker</strong>…"),
                duration=None,
                close_button=False,
                id="disconnecting_docker"
            )
            photfun_client.disconnect_docker()
            ui.notification_remove("disconnecting_docker")
            # Mensaje conciso y verde tras desconexión
            ui.notification_show(
                ui.HTML("<strong>Disconnected</strong>"),
                type="message",
                duration=2,
                id="disconnected"
            )
            docker_toggle_handler()
        else:
            # Mensaje pequeño y gris mientras conecta
            ui.notification_show(
                ui.HTML("Connecting <strong>Docker</strong>…"),
                duration=None,
                close_button=False,
                id="connecting_docker"
            )
            photfun_client.reconnect_docker()
            ui.notification_remove("connecting_docker")
            if photfun_client.docker_container[0]:
                # Mensaje conciso y verde tras conexión
                ui.notification_show(
                    ui.HTML("<strong>Connected</strong>"),
                    type="message",
                    duration=2,
                    id="connected"
                )
            else:
                # Mensaje conciso y rojo si falla
                ui.notification_show(
                    ui.HTML("<strong>Failed</strong>"),
                    type="error",
                    duration=2,
                    id="failed"
                )
            docker_toggle_handler()


